@@ -77,6 +77,20 @@ def get_photos_db(*db_options):
         return None
 
 
+class DateTimeISO8601(click.ParamType):
+
+    name = "DATETIME"
+
+    def convert(self, value, param, ctx):
+        try:
+            return datetime.datetime.fromisoformat(value)
+        except:
+            self.fail(
+                f"Invalid value for --{param.name}: invalid datetime format {value}. "
+                "Valid format: YYYY-MM-DD[*HH[:MM[:SS[.fff[fff]]]][+HH:MM[:SS[.ffffff]]]]"
+            )
+
+
 # Click CLI object & context settings
 class CLI_Obj:
     def __init__(self, db=None, json=False, debug=False):
@@ -298,6 +312,15 @@ def query_options(f):
             multiple=True,
             help="Search for photos with UUID(s).",
         ),
+        o(
+            "--uuid-from-file",
+            metavar="FILE",
+            default=None,
+            multiple=False,
+            help="Search for photos with UUID(s) loaded from FILE. "
+            "Format is a single UUID per line.  Lines preceeded with # are ignored.",
+            type=click.Path(exists=True),
+        ),
         o(
             "--title",
             metavar="TITLE",
@@ -445,13 +468,13 @@ def query_options(f):
         ),
         o(
             "--from-date",
-            help="Search by start item date, e.g. 2000-01-12T12:00:00 or 2000-12-31 (ISO 8601 w/o TZ).",
-            type=click.DateTime(),
+            help="Search by start item date, e.g. 2000-01-12T12:00:00, 2001-01-12T12:00:00-07:00, or 2000-12-31 (ISO 8601).",
+            type=DateTimeISO8601(),
         ),
         o(
             "--to-date",
-            help="Search by end item date, e.g. 2000-01-12T12:00:00 or 2000-12-31 (ISO 8601 w/o TZ).",
-            type=click.DateTime(),
+            help="Search by end item date, e.g. 2000-01-12T12:00:00, 2001-01-12T12:00:00-07:00, or 2000-12-31 (ISO 8601).",
+            type=DateTimeISO8601(),
         ),
     ]
     for o in options[::-1]:
@@ -891,6 +914,7 @@ def query(
     album,
     folder,
     uuid,
+    uuid_from_file,
     title,
     no_title,
     description,
@@ -954,6 +978,7 @@ def query(
         album,
         folder,
         uuid,
+        uuid_from_file,
         edited,
         external_edit,
         uti,
@@ -998,6 +1023,12 @@ def query(
     if only_photos:
         ismovie = False
 
+    # load UUIDs if necessary and append to any uuids passed with --uuid
+    if uuid_from_file:
+        uuid_list = list(uuid)  # Click option is a tuple
+        uuid_list.extend(load_uuid_from_file(uuid_from_file))
+        uuid = tuple(uuid_list)
+
     # below needed for to make CliRunner work for testing
     cli_db = cli_obj.db if cli_obj is not None else None
     db = get_photos_db(*photos_library, db, cli_db)
@@ -1090,6 +1121,11 @@ def query(
     help="Hardlink files instead of copying them.  "
     "Cannot be used with --exiftool which creates copies of the files with embedded EXIF data.",
 )
+@click.option(
+    "--touch-file",
+    is_flag=True,
+    help="Sets the file's modification time to match photo date.",
+)
 @click.option(
     "--overwrite",
     is_flag=True,
@@ -1233,6 +1269,13 @@ def query(
     "to a filesystem that doesn't support Mac OS extended attributes.  Only use this if you get "
     "an error while exporting.",
 )
+@click.option(
+    "--use-photos-export",
+    is_flag=True,
+    default=False,
+    hidden=True,
+    help="Force the use of AppleScript to export even if not missing (see also --download-missing).",
+)
 @DB_ARGUMENT
 @click.argument("dest", nargs=1, type=click.Path(exists=True))
 @click.pass_obj
@@ -1247,6 +1290,7 @@ def export(
     album,
     folder,
     uuid,
+    uuid_from_file,
     title,
     no_title,
     description,
@@ -1267,6 +1311,7 @@ def export(
     update,
     dry_run,
     export_as_hardlink,
+    touch_file,
     overwrite,
     export_by_date,
     skip_edited,
@@ -1312,6 +1357,7 @@ def export(
     label,
     deleted,
     deleted_only,
+    use_photos_export,
 ):
     """ Export photos from the Photos database.
         Export path DEST is required.
@@ -1329,7 +1375,7 @@ def export(
     VERBOSE = True if verbose_ else False
 
     if not os.path.isdir(dest):
-        sys.exit("DEST must be valid path")
+        sys.exit(f"DEST {dest} must be valid path")
 
     # sanity check input args
     exclusive = [
@@ -1381,6 +1427,12 @@ def export(
     if only_photos:
         ismovie = False
 
+    # load UUIDs if necessary and append to any uuids passed with --uuid
+    if uuid_from_file:
+        uuid_list = list(uuid)  # Click option is a tuple
+        uuid_list.extend(load_uuid_from_file(uuid_from_file))
+        uuid = tuple(uuid_list)
+
     # below needed for to make CliRunner work for testing
     cli_db = cli_obj.db if cli_obj is not None else None
     db = get_photos_db(*photos_library, db, cli_db)
@@ -1476,7 +1528,6 @@ def export(
         deleted_only=deleted_only,
     )
 
-    results_exported = []
     if photos:
         if export_bursts:
             # add the burst_photos to the export set
@@ -1495,10 +1546,12 @@ def export(
         # because the original code used --original-name as an option
         original_name = not current_name
 
+        results_exported = []
         results_new = []
         results_updated = []
         results_skipped = []
         results_exif_updated = []
+        results_touched = []
         if verbose_:
             for p in photos:
                 results = export_photo(
@@ -1526,13 +1579,16 @@ def export(
                     export_db=export_db,
                     fileutil=fileutil,
                     dry_run=dry_run,
+                    touch_file=touch_file,
                     edited_suffix=edited_suffix,
+                    use_photos_export=use_photos_export,
                 )
                 results_exported.extend(results.exported)
                 results_new.extend(results.new)
                 results_updated.extend(results.updated)
                 results_skipped.extend(results.skipped)
                 results_exif_updated.extend(results.exif_updated)
+                results_touched.extend(results.touched)
 
         else:
             # show progress bar
@@ -1563,31 +1619,42 @@ def export(
                         export_db=export_db,
                         fileutil=fileutil,
                         dry_run=dry_run,
+                        touch_file=touch_file,
                         edited_suffix=edited_suffix,
+                        use_photos_export=use_photos_export,
                     )
                     results_exported.extend(results.exported)
                     results_new.extend(results.new)
                     results_updated.extend(results.updated)
                     results_skipped.extend(results.skipped)
                     results_exif_updated.extend(results.exif_updated)
+                    results_touched.extend(results.touched)
         stop_time = time.perf_counter()
         # print summary results
         if update:
             photo_str_new = "photos" if len(results_new) != 1 else "photo"
-            photo_str_updated = "photos" if len(results_new) != 1 else "photo"
+            photo_str_updated = "photos" if len(results_updated) != 1 else "photo"
             photo_str_skipped = "photos" if len(results_skipped) != 1 else "photo"
+            photo_str_touched = "photos" if len(results_touched) != 1 else "photo"
             photo_str_exif_updated = (
                 "photos" if len(results_exif_updated) != 1 else "photo"
             )
-            click.echo(
+            summary = (
                 f"Exported: {len(results_new)} {photo_str_new}, "
-                + f"updated: {len(results_updated)} {photo_str_updated}, "
-                + f"skipped: {len(results_skipped)} {photo_str_skipped}, "
-                + f"updated EXIF data: {len(results_exif_updated)} {photo_str_exif_updated}"
+                f"updated: {len(results_updated)} {photo_str_updated}, "
+                f"skipped: {len(results_skipped)} {photo_str_skipped}, "
+                f"updated EXIF data: {len(results_exif_updated)} {photo_str_exif_updated}"
             )
+            if touch_file:
+                summary += f", touched date: {len(results_touched)} {photo_str_touched}"
+            click.echo(summary)
         else:
             photo_str = "photos" if len(results_exported) != 1 else "photo"
-            click.echo(f"Exported: {len(results_exported)} {photo_str}")
+            photo_str_touched = "photos" if len(results_touched) != 1 else "photo"
+            summary = f"Exported: {len(results_exported)} {photo_str}"
+            if touch_file:
+                summary += f", touched date: {len(results_touched)} {photo_str_touched}"
+            click.echo(summary)
         click.echo(f"Elapsed time: {(stop_time-start_time):.3f} seconds")
     else:
         click.echo("Did not find any photos to export")
@@ -2035,7 +2102,9 @@ def export_photo(
     export_db=None,
     fileutil=FileUtil,
     dry_run=None,
+    touch_file=None,
     edited_suffix="_edited",
+    use_photos_export=False,
 ):
     """ Helper function for export that does the actual export
 
@@ -2063,6 +2132,8 @@ def export_photo(
         export_db: export database instance compatible with ExportDB_ABC
         fileutil: file util class compatible with FileUtilABC
         dry_run: boolean; if True, doesn't actually export or update any files
+        touch_file: boolean; sets file's modification time to match photo date
+        use_photos_export: boolean; if True forces the use of AppleScript to export even if photo not missing
 
     Returns:
         list of path(s) of exported photo or None if photo was missing
@@ -2077,19 +2148,26 @@ def export_photo(
         if photo.ismissing:
             space = " " if not verbose_ else ""
             verbose(f"{space}Skipping missing photo {photo.filename}")
-            return ExportResults([], [], [], [], [])
+            return ExportResults([], [], [], [], [], [])
         elif not os.path.exists(photo.path):
             space = " " if not verbose_ else ""
             verbose(
                 f"{space}WARNING: file {photo.path} is missing but ismissing=False, "
                 f"skipping {photo.filename}"
             )
-            return ExportResults([], [], [], [], [])
+            return ExportResults([], [], [], [], [], [])
     elif photo.ismissing and not photo.iscloudasset or not photo.incloud:
         verbose(
             f"Skipping missing {photo.filename}: not iCloud asset or missing from cloud"
         )
-        return ExportResults([], [], [], [], [])
+        return ExportResults([], [], [], [], [], [])
+
+    results_exported = []
+    results_new = []
+    results_updated = []
+    results_skipped = []
+    results_exif_updated = []
+    results_touched = []
 
     filenames = get_filenames_from_template(photo, filename_template, original_name)
     for filename in filenames:
@@ -2108,16 +2186,13 @@ def export_photo(
 
         # if download_missing and the photo is missing or path doesn't exist,
         # try to download with Photos
-        use_photos_export = download_missing and (
-            photo.ismissing or not os.path.exists(photo.path)
+        use_photos_export = (
+            download_missing and (photo.ismissing or not os.path.exists(photo.path))
+            if not use_photos_export
+            else True
         )
 
         # export the photo to each path in dest_paths
-        results_exported = []
-        results_new = []
-        results_updated = []
-        results_skipped = []
-        results_exif_updated = []
         for dest_path in dest_paths:
             export_results = photo.export2(
                 dest_path,
@@ -2139,6 +2214,7 @@ def export_photo(
                 export_db=export_db,
                 fileutil=fileutil,
                 dry_run=dry_run,
+                touch_file=touch_file,
             )
 
             results_exported.extend(export_results.exported)
@@ -2146,6 +2222,7 @@ def export_photo(
             results_updated.extend(export_results.updated)
             results_skipped.extend(export_results.skipped)
             results_exif_updated.extend(export_results.exif_updated)
+            results_touched.extend(export_results.touched)
 
             if verbose_:
                 for exported in export_results.exported:
@@ -2156,13 +2233,14 @@ def export_photo(
                     verbose(f"Exported updated file {updated}")
                 for skipped in export_results.skipped:
                     verbose(f"Skipped up to date file {skipped}")
+                for touched in export_results.touched:
+                    verbose(f"Touched date on file {touched}")
 
             # if export-edited, also export the edited version
             # verify the photo has adjustments and valid path to avoid raising an exception
             if export_edited and photo.hasadjustments:
                 # if download_missing and the photo is missing or path doesn't exist,
                 # try to download with Photos
-                use_photos_export = download_missing and photo.path_edited is None
                 if not download_missing and photo.path_edited is None:
                     verbose(f"Skipping missing edited photo for {filename}")
                 else:
@@ -2195,6 +2273,7 @@ def export_photo(
                         export_db=export_db,
                         fileutil=fileutil,
                         dry_run=dry_run,
+                        touch_file=touch_file,
                     )
 
                     results_exported.extend(export_results_edited.exported)
@@ -2202,6 +2281,7 @@ def export_photo(
                     results_updated.extend(export_results_edited.updated)
                     results_skipped.extend(export_results_edited.skipped)
                     results_exif_updated.extend(export_results_edited.exif_updated)
+                    results_touched.extend(export_results_edited.touched)
 
                     if verbose_:
                         for exported in export_results_edited.exported:
@@ -2212,6 +2292,8 @@ def export_photo(
                             verbose(f"Exported updated file {updated}")
                         for skipped in export_results_edited.skipped:
                             verbose(f"Skipped up to date file {skipped}")
+                        for touched in export_results_edited.touched:
+                            verbose(f"Touched date on file {touched}")
 
     return ExportResults(
         results_exported,
@@ -2219,6 +2301,7 @@ def export_photo(
         results_updated,
         results_skipped,
         results_exif_updated,
+        results_touched,
     )
 
 
@@ -2339,5 +2422,32 @@ def find_files_in_branch(pathname, filename):
     return files
 
 
+def load_uuid_from_file(filename):
+    """ Load UUIDs from file.  Does not validate UUIDs.
+        Format is 1 UUID per line, any line beginning with # is ignored.
+        Whitespace is stripped.
+
+    Arguments:
+        filename: file name of the file containing UUIDs
+    
+    Returns:
+        list of UUIDs or empty list of no UUIDs in file
+    
+    Raises:
+        FileNotFoundError if file does not exist
+    """
+
+    if not pathlib.Path(filename).is_file():
+        raise FileNotFoundError(f"Could not find file {filename}")
+
+    uuid = []
+    with open(filename, "r") as uuid_file:
+        for line in uuid_file:
+            line = line.strip()
+            if len(line) and line[0] != "#":
+                uuid.append(line)
+    return uuid
+
+
 if __name__ == "__main__":
     cli()  # pylint: disable=no-value-for-parameter
